@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2007-2010 Open Source Applications Foundation.
+ * Copyright (c) 2007-2025 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,8 @@
 #include "charset.h"
 #include "macros.h"
 
+#include "arg.h"
+
 
 /* CharsetDetector */
 
@@ -118,7 +120,7 @@ static int t_charsetdetector_init(t_charsetdetector *self,
 {
     const char *text;
     charsArg encoding;
-    int textSize;
+    size_t textSize;
 
     switch (PyTuple_Size(args)) {
       case 0:
@@ -126,7 +128,7 @@ static int t_charsetdetector_init(t_charsetdetector *self,
         break;
 
       case 1:
-        if (!parseArgs(args, "k", &text, &textSize))
+        if (!parseArgs(args, arg::k(&text, &textSize)))
         {
             INT_STATUS_CALL(self->object = ucsdet_open(&status));
             INT_STATUS_CALL(ucsdet_setText(self->object, text, textSize,
@@ -139,7 +141,7 @@ static int t_charsetdetector_init(t_charsetdetector *self,
         return -1;
 
       case 2:
-        if (!parseArgs(args, "kn", &text, &textSize, &encoding))
+        if (!parseArgs(args, arg::k(&text, &textSize), arg::n(&encoding)))
         {
             INT_STATUS_CALL(self->object = ucsdet_open(&status));
             INT_STATUS_CALL(ucsdet_setText(self->object, text,
@@ -168,12 +170,12 @@ static PyObject *t_charsetdetector_setText(t_charsetdetector *self,
                                            PyObject *arg)
 {
     const char *text;
-    int size;
+    size_t len;
 
-    if (!parseArg(arg, "k", &text, &size))
+    if (!parseArg(arg, arg::k(&text, &len)))
     {
         /* ref'd */
-        STATUS_CALL(ucsdet_setText(self->object, text, size, &status));
+        STATUS_CALL(ucsdet_setText(self->object, text, len, &status));
 
         Py_INCREF(arg);
         Py_XDECREF(self->text);
@@ -189,12 +191,12 @@ static PyObject *t_charsetdetector_setDeclaredEncoding(t_charsetdetector *self,
                                                        PyObject *arg)
 {
     const char *encoding;
-    int size;
+    size_t len;
 
-    if (!parseArg(arg, "k", &encoding, &size))
+    if (!parseArg(arg, arg::k(&encoding, &len)))
     {
         /* copied */
-        STATUS_CALL(ucsdet_setDeclaredEncoding(self->object, encoding, size,
+        STATUS_CALL(ucsdet_setDeclaredEncoding(self->object, encoding, len,
                                                &status));
         Py_RETURN_NONE;
     }
@@ -251,9 +253,9 @@ static PyObject *t_charsetdetector_detectAll(t_charsetdetector *self)
 static PyObject *t_charsetdetector_enableInputFilter(t_charsetdetector *self,
                                                      PyObject *arg)
 {
-    int filter;
+    UBool filter;
 
-    if (!parseArg(arg, "B", &filter))
+    if (!parseArg(arg, arg::B(&filter)))
     {
         UBool previous = ucsdet_enableInputFilter(self->object, filter);
         Py_RETURN_BOOL(previous);
@@ -311,26 +313,16 @@ static PyObject *t_charsetmatch_str(t_charsetmatch *self)
     {
         UErrorCode status = U_ZERO_ERROR;
         int size = (int) PyBytes_GET_SIZE(self->detector->text);
-        UChar *buf = new UChar[size];
-        PyObject *u;
+        std::unique_ptr<UChar[]> buf(new UChar[size]);
 
-        if (!buf)
-        {
-            PyErr_SetNone(PyExc_MemoryError);
-            return NULL;
-        }
+        if (!buf.get())
+            return PyErr_NoMemory();
 
-        size = ucsdet_getUChars(self->object, buf, size, &status);
+        size = ucsdet_getUChars(self->object, buf.get(), size, &status);
         if (U_FAILURE(status))
-        {
-            delete[] buf;
             return ICUException(status).reportError();
-        }
 
-        u = PyUnicode_FromUnicodeString(buf, size);
-        delete[] buf;
-
-        return u;
+        return PyUnicode_FromUnicodeString(buf.get(), size);
     }
 
 #if PY_VERSION_HEX < 0x03030000 || defined(PYPY_VERSION)
