@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2004-2022 Open Source Applications Foundation.
+ * Copyright (c) 2004-2025 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,8 @@
 #include "measureunit.h"
 #include "macros.h"
 
+#include "arg.h"
+
 #if U_ICU_VERSION_HEX >= 0x04020000
 DECLARE_CONSTANTS_TYPE(UTimeUnitFields)
 #endif
@@ -295,6 +297,12 @@ static PyObject *t_measureunit_createBeaufort(PyTypeObject *type);
 static PyObject *t_measureunit_createGasolineEnergyDensity(PyTypeObject *type);
 #endif
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(77, 0, 0)
+static PyObject *t_measureunit_createPortionPer1E9(PyTypeObject *type);
+static PyObject *t_measureunit_withConstantDenominator(t_measureunit *self, PyObject *arg);
+static PyObject *t_measureunit_getConstantDenominator(t_measureunit *self);
+#endif
+
 static PyMethodDef t_measureunit_methods[] = {
 #if U_ICU_VERSION_HEX >= VERSION_HEX(53, 0, 0)
     DECLARE_METHOD(t_measureunit, getType, METH_NOARGS),
@@ -534,12 +542,17 @@ static PyMethodDef t_measureunit_methods[] = {
 #endif
 #if U_ICU_VERSION_HEX >= VERSION_HEX(74, 0, 0)
     DECLARE_METHOD(t_measureunit, createGasolineEnergyDensity, METH_NOARGS | METH_CLASS),
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(77, 0, 0)
+    DECLARE_METHOD(t_measureunit, createPortionPer1E9, METH_NOARGS | METH_CLASS),
+    DECLARE_METHOD(t_measureunit, withConstantDenominator, METH_O),
+    DECLARE_METHOD(t_measureunit, getConstantDenominator, METH_NOARGS),
 #endif
     { NULL, NULL, 0, NULL }
 };
 
-DECLARE_TYPE(MeasureUnit, t_measureunit, UObject, MeasureUnit,
-             abstract_init, NULL)
+DECLARE_DEALLOC_TYPE(MeasureUnit, t_measureunit, UObject,
+                     MeasureUnit, abstract_init)
 
 
 /* Measure */
@@ -562,9 +575,9 @@ static PyMethodDef t_measure_methods[] = {
 };
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(53, 1, 0)
-DECLARE_TYPE(Measure, t_measure, UObject, Measure, t_measure_init, NULL)
+DECLARE_DEALLOC_TYPE(Measure, t_measure, UObject, Measure, t_measure_init)
 #else
-DECLARE_TYPE(Measure, t_measure, UObject, Measure, abstract_init, NULL)
+DECLARE_DEALLOC_TYPE(Measure, t_measure, UObject, Measure, abstract_init)
 #endif
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(60, 0, 0)
@@ -593,8 +606,7 @@ static PyMethodDef t_nounit_methods[] = {
     { NULL, NULL, 0, NULL }
 };
 
-DECLARE_TYPE(NoUnit, t_nounit, MeasureUnit, NoUnit_,
-             abstract_init, NULL)
+DECLARE_DEALLOC_TYPE(NoUnit, t_nounit, MeasureUnit, NoUnit_, abstract_init)
 
 #endif
 
@@ -617,8 +629,8 @@ static PyMethodDef t_currencyunit_methods[] = {
     { NULL, NULL, 0, NULL }
 };
 
-DECLARE_TYPE(CurrencyUnit, t_currencyunit, MeasureUnit, CurrencyUnit,
-             t_currencyunit_init, NULL)
+DECLARE_DEALLOC_TYPE(CurrencyUnit, t_currencyunit, MeasureUnit,
+                     CurrencyUnit, t_currencyunit_init)
 
 
 /* CurrencyAmount */
@@ -639,8 +651,8 @@ static PyMethodDef t_currencyamount_methods[] = {
     { NULL, NULL, 0, NULL }
 };
 
-DECLARE_TYPE(CurrencyAmount, t_currencyamount, Measure, CurrencyAmount,
-             t_currencyamount_init, NULL)
+DECLARE_DEALLOC_TYPE(CurrencyAmount, t_currencyamount, Measure,
+                     CurrencyAmount, t_currencyamount_init)
 
 
 /* TimeUnit */
@@ -661,8 +673,7 @@ static PyMethodDef t_timeunit_methods[] = {
     { NULL, NULL, 0, NULL }
 };
 
-DECLARE_TYPE(TimeUnit, t_timeunit, MeasureUnit, TimeUnit,
-             abstract_init, NULL)
+DECLARE_DEALLOC_TYPE(TimeUnit, t_timeunit, MeasureUnit, TimeUnit, abstract_init)
 
 
 /* TimeUnitAmount */
@@ -683,15 +694,15 @@ static PyMethodDef t_timeunitamount_methods[] = {
     { NULL, NULL, 0, NULL }
 };
 
-DECLARE_TYPE(TimeUnitAmount, t_timeunitamount, Measure, TimeUnitAmount,
-             t_timeunitamount_init, NULL)
+DECLARE_DEALLOC_TYPE(TimeUnitAmount, t_timeunitamount, Measure,
+                     TimeUnitAmount, t_timeunitamount_init)
 
 #endif
 
 
 /* MeasureUnit */
 
-DEFINE_RICHCMP(MeasureUnit, t_measureunit)
+DEFINE_RICHCMP__ARG__(MeasureUnit, t_measureunit)
 
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(53, 0, 0)
@@ -719,7 +730,7 @@ static PyObject *t_measureunit_getAvailable(PyTypeObject *type, PyObject *arg)
 {
     charsArg type_name;
 
-    if (!parseArg(arg, "n", &type_name))
+    if (!parseArg(arg, arg::n(&type_name)))
     {
         UErrorCode status = U_ZERO_ERROR;
         int32_t size = MeasureUnit::getAvailable(type_name, NULL, 0, status);
@@ -729,16 +740,15 @@ static PyObject *t_measureunit_getAvailable(PyTypeObject *type, PyObject *arg)
             if (status != U_BUFFER_OVERFLOW_ERROR)
                 return ICUException(status).reportError();
 
-            MeasureUnit *mus = new MeasureUnit[size];
+            std::unique_ptr<MeasureUnit[]> mus(new MeasureUnit[size]);
+            if (!mus.get())
+                return PyErr_NoMemory();
 
             status = U_ZERO_ERROR;
-            size = MeasureUnit::getAvailable(type_name, mus, size, status);
+            size = MeasureUnit::getAvailable(type_name, mus.get(), size, status);
 
             if (U_FAILURE(status))
-            {
-                delete[] mus;
                 return ICUException(status).reportError();
-            }
 
             PyObject *result = PyTuple_New(size);
 
@@ -746,8 +756,6 @@ static PyObject *t_measureunit_getAvailable(PyTypeObject *type, PyObject *arg)
                 PyTuple_SET_ITEM(result, i, wrap_MeasureUnit(
                     (MeasureUnit *) mus[i].clone(), T_OWNED));
 
-            delete[] mus;
-
             return result;
         }
     }
@@ -791,7 +799,7 @@ static PyObject *t_measureunit_product(t_measureunit *self, PyObject *arg)
 {
     MeasureUnit *other;
 
-    if (!parseArg(arg, "P", TYPE_ID(MeasureUnit), &other))
+    if (!parseArg(arg, arg::P<MeasureUnit>(TYPE_ID(MeasureUnit), &other)))
     {
         MeasureUnit mu;
         STATUS_CALL(mu = self->object->product(*other, status));
@@ -817,8 +825,8 @@ static PyObject *t_measureunit___truediv__(PyObject *arg0, PyObject *arg1)
     MeasureUnit *mu0, *mu1;
     int i;
 
-    if (!parseArg(arg0, "P", TYPE_ID(MeasureUnit), &mu0) &&
-        !parseArg(arg1, "P", TYPE_ID(MeasureUnit), &mu1))
+    if (!parseArg(arg0, arg::P<MeasureUnit>(TYPE_ID(MeasureUnit), &mu0)) &&
+        !parseArg(arg1, arg::P<MeasureUnit>(TYPE_ID(MeasureUnit), &mu1)))
     {
         MeasureUnit mu;
 
@@ -828,8 +836,8 @@ static PyObject *t_measureunit___truediv__(PyObject *arg0, PyObject *arg1)
         return wrap_MeasureUnit((MeasureUnit *) mu.clone(), T_OWNED);
     }
 
-    if (!parseArg(arg0, "i", &i) && i == 1 &&
-        !parseArg(arg1, "P", TYPE_ID(MeasureUnit), &mu0))
+    if (!parseArg(arg0, arg::i(&i)) && i == 1 &&
+        !parseArg(arg1, arg::P<MeasureUnit>(TYPE_ID(MeasureUnit), &mu0)))
     {
         MeasureUnit mu;
 
@@ -847,8 +855,8 @@ static PyObject *t_measureunit___pow__(PyObject *arg0,
     MeasureUnit *mu0;
     int p;
 
-    if (!parseArg(arg0, "P", TYPE_ID(MeasureUnit), &mu0) &&
-        !parseArg(arg1, "i", &p) && p != 0 && arg2 == Py_None)
+    if (!parseArg(arg0, arg::P<MeasureUnit>(TYPE_ID(MeasureUnit), &mu0)) &&
+        !parseArg(arg1, arg::i(&p)) && p != 0 && arg2 == Py_None)
     {
         MeasureUnit mu = *mu0;
 
@@ -882,7 +890,7 @@ static PyObject *t_measureunit_forIdentifier(PyTypeObject *type, PyObject *arg)
 {
     charsArg identifier;
 
-    if (!parseArg(arg, "n", &identifier))
+    if (!parseArg(arg, arg::n(&identifier)))
     {
         MeasureUnit mu;
         STATUS_CALL(mu = MeasureUnit::forIdentifier(
@@ -902,7 +910,7 @@ static PyObject *t_measureunit_withPrefix(t_measureunit *self, PyObject *arg)
 {
     UMeasurePrefix prefix;
 
-    if (!parseArg(arg, "i", &prefix))
+    if (!parseArg(arg, arg::Enum<UMeasurePrefix>(&prefix)))
     {
         MeasureUnit mu;
         STATUS_CALL(mu = self->object->withPrefix(prefix, status));
@@ -1166,6 +1174,10 @@ createMU(Beaufort)
 createMU(GasolineEnergyDensity)
 #endif
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(77, 0, 0)
+createMU(PortionPer1E9)
+#endif
+
 /* Measure */
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(53, 1, 0)
@@ -1178,15 +1190,18 @@ static int t_measure_init(t_measure *self, PyObject *args, PyObject *kwds)
 
     switch (PyTuple_Size(args)) {
       case 2:
-        if (!parseArgs(args, "dP", TYPE_ID(MeasureUnit), &d, &unit))
+        if (!parseArgs(args,
+                       arg::d(&d),
+                       arg::P<MeasureUnit>(TYPE_ID(MeasureUnit), &unit)))
         {
             INT_STATUS_CALL(self->object = new Measure(
                 Formattable(d), (MeasureUnit *) unit->clone(), status));
             self->flags = T_OWNED;
             break;
         }
-        if (!parseArgs(args, "PP", TYPE_CLASSID(Formattable),
-                       TYPE_ID(MeasureUnit), &number, &unit))
+        if (!parseArgs(args,
+                       arg::P<Formattable>(TYPE_CLASSID(Formattable), &number),
+                       arg::P<MeasureUnit>(TYPE_ID(MeasureUnit), &unit)))
         {
             INT_STATUS_CALL(self->object = new Measure(
                 *number, (MeasureUnit *) unit->clone(), status));
@@ -1266,7 +1281,33 @@ static PyObject *t_measure_str(t_measure *self)
 #endif
 }
 
-DEFINE_RICHCMP(Measure, t_measure)
+#if U_ICU_VERSION_HEX >= VERSION_HEX(77, 0, 0)
+
+static PyObject *t_measureunit_withConstantDenominator(t_measureunit *self, PyObject *arg)
+{
+    PY_LONG_LONG denominator;
+
+    if (!parseArg(arg, arg::L(&denominator)))
+    {
+        MeasureUnit mu;
+        STATUS_CALL(mu = self->object->withConstantDenominator(denominator, status));
+        return wrap_MeasureUnit((MeasureUnit *) mu.clone(), T_OWNED);
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "withConstantDenominator", arg);
+}
+
+static PyObject *t_measureunit_getConstantDenominator(t_measureunit *self)
+{
+    PY_LONG_LONG denominator;
+
+    STATUS_CALL(denominator = self->object->getConstantDenominator(status));
+    return PyLong_FromLongLong(denominator);
+}
+    
+#endif // ICU 77
+
+DEFINE_RICHCMP__ARG__(Measure, t_measure)
 
 
 /* NoUnit */
@@ -1297,10 +1338,9 @@ static int t_currencyunit_init(t_currencyunit *self,
                                PyObject *args, PyObject *kwds)
 {
     UErrorCode status = U_ZERO_ERROR;
-    UnicodeString *u;
-    UnicodeString _u;
+    UnicodeString *u, _u;
 
-    if (!parseArgs(args, "S", &u, &_u))
+    if (!parseArgs(args, arg::S(&u, &_u)))
     {
         CurrencyUnit *cu = new CurrencyUnit(u->getTerminatedBuffer(), status);
 
@@ -1342,7 +1382,7 @@ static PyObject *t_currencyunit_getName(t_currencyunit *self, PyObject *args)
         return PyUnicode_FromUnicodeString(name, len);
         
       case 1:
-        if (!parseArgs(args, "P", TYPE_CLASSID(Locale), &locale))
+        if (!parseArgs(args, arg::P<Locale>(TYPE_CLASSID(Locale), &locale)))
         {
             STATUS_CALL(name = ucurr_getName(
                 isoCode, locale->getName(), style,
@@ -1352,7 +1392,9 @@ static PyObject *t_currencyunit_getName(t_currencyunit *self, PyObject *args)
         break;
 
       case 2:
-        if (!parseArgs(args, "Pi", TYPE_CLASSID(Locale), &locale, &style))
+        if (!parseArgs(args,
+                       arg::P<Locale>(TYPE_CLASSID(Locale), &locale),
+                       arg::Enum<UCurrNameStyle>(&style)))
         {
             STATUS_CALL(name = ucurr_getName(
                 isoCode, locale->getName(), style,
@@ -1380,11 +1422,11 @@ static int t_currencyamount_init(t_currencyamount *self,
     UErrorCode status = U_ZERO_ERROR;
     Formattable *f;
     double d;
-    UnicodeString *u;
-    UnicodeString _u;
+    UnicodeString *u, _u;
 
-    if (!parseArgs(args, "PS", TYPE_CLASSID(Formattable),
-                   &f, &u, &_u))
+    if (!parseArgs(args,
+                   arg::P<Formattable>(TYPE_CLASSID(Formattable), &f),
+                   arg::S(&u, &_u)))
     {
         CurrencyAmount *ca =
             new CurrencyAmount(*f, u->getTerminatedBuffer(), status);
@@ -1401,7 +1443,7 @@ static int t_currencyamount_init(t_currencyamount *self,
         return 0;
     }
 
-    if (!parseArgs(args, "dS", &d, &u, &_u))
+    if (!parseArgs(args, arg::d(&d), arg::S(&u, &_u)))
     {
         CurrencyAmount *ca =
             new CurrencyAmount(d, u->getTerminatedBuffer(), status);
@@ -1469,7 +1511,7 @@ static PyObject *t_timeunit_createInstance(PyTypeObject *type, PyObject *arg)
 {
     TimeUnit::UTimeUnitFields field;
 
-    if (!parseArg(arg, "i", &field))
+    if (!parseArg(arg, arg::Enum<TimeUnit::UTimeUnitFields>(&field)))
     {
         TimeUnit *tu;
         STATUS_CALL(tu = TimeUnit::createInstance(field, status));
@@ -1492,14 +1534,18 @@ static int t_timeunitamount_init(t_timeunitamount *self, PyObject *args,
 
     switch (PyTuple_Size(args)) {
       case 2:
-        if (!parseArgs(args, "Pi", TYPE_CLASSID(Formattable), &obj, &field))
+        if (!parseArgs(args,
+                       arg::P<Formattable>(TYPE_CLASSID(Formattable), &obj),
+                       arg::Enum<TimeUnit::UTimeUnitFields>(&field)))
         {
             INT_STATUS_CALL(self->object = new TimeUnitAmount(
                 *obj, field, status));
             self->flags = T_OWNED;
             break;
         }
-        if (!parseArgs(args, "di", &d, &field))
+        if (!parseArgs(args,
+                       arg::d(&d),
+                       arg::Enum<TimeUnit::UTimeUnitFields>(&field)))
         {
             INT_STATUS_CALL(self->object = new TimeUnitAmount(
                 d, field, status));
